/*
 * derdump -- display a DER encoded data structure 
 *
 * (c) 2001 Dr. Andreas Mueller, Beratung und Entwicklung
 *
 * $Id: derdump.c,v 1.2 2001/03/12 08:20:02 afm Exp $
 */
#include <openssl/asn1.h>
#include <openssl/err.h>
#include <init.h>

extern int	optind;
extern char	*optarg;

int	base64 = 0;

int	main(int argc, char *argv[]) {
	BIO		*inbio, *outbio, *b64 = NULL, *readbio, *membio;
	int		bytes = 0, length, indent = 1, c;
	unsigned char	*data;
	char		*filename = NULL;

	/* parse command line arguments					*/
	while ((c = getopt(argc, argv, "i")) != EOF)
		switch (c) {
		case 'i':
			indent = 0;
			break;
		case 'b':
			b64 = BIO_new(BIO_f_base64());
			break;
		}

	/* initialize the openssl libraries as with scepdump, to add	*/
	/* the scep specific object ids					*/
	scepinit();

	/* check for a filename argument				*/
	if (argc - optind > 0)
		filename = argv[optind];

	/* open input file or read from standard input			*/
	inbio = BIO_new(BIO_s_file());
	if (filename) {
		if (0 >= BIO_read_filename(inbio, filename)) {
			ERR_print_errors(bio_err);
			exit(EXIT_FAILURE);
		}
	} else {
		BIO_set_fp(inbio, stdin, BIO_NOCLOSE | BIO_FP_TEXT);
	}
	outbio = BIO_new(BIO_s_file());
	BIO_set_fp(outbio, stdout, BIO_NOCLOSE | BIO_FP_TEXT);

	/* read PEM or base 64 data					*/
	if (b64 != NULL) {
		readbio = BIO_push(inbio, b64);
	} else {
		readbio = inbio;
	}

	/* create a memory bio to read the input into			*/
	membio = BIO_new(BIO_s_mem());
	do {
		unsigned char	buffer[1024];
		bytes = BIO_read(inbio, buffer, sizeof(buffer));
		if (bytes > 0)
			BIO_write(membio, buffer, bytes);
	} while (bytes > 0);
	BIO_flush(membio);

	/* get the memory data from the memory BIO			*/
	length = BIO_get_mem_data(membio, &data);

	/* turn the memory over to us					*/
	BIO_set_flags(membio, BIO_FLAGS_MEM_RDONLY);

	/* recursively display all data					*/
	ASN1_parse_dump(outbio, data, length, indent, 1);

	/* that's it							*/
	exit(EXIT_SUCCESS);
}
